@@ -2,7 +2,7 @@
 //   Apptainer a Series of LF Projects LLC.
 //   For website terms of use, trademark policy, privacy policy and other
 //   project policies see https://lfprojects.org/policies
-// Copyright (c) 2018-2022, Sylabs Inc. All rights reserved.
+// Copyright (c) 2018-2024, Sylabs Inc. All rights reserved.
 // This software is licensed under a 3-clause BSD license. Please consult the
 // LICENSE.md file distributed with the sources of this project regarding your
 // rights to use or distribute this software.
@@ -17,28 +17,28 @@ import (
 
 // actionflags.go contains flag variables for action-like commands to draw from
 var (
-	appName          string
-	bindPaths        []string
-	mounts           []string
-	homePath         string
-	overlayPath      []string
-	scratchPath      []string
-	workdirPath      string
-	cwdPath          string
-	shellPath        string
-	hostname         string
-	network          string
-	networkArgs      []string
-	dns              string
-	security         []string
-	cgroupsTOMLFile  string
-	containLibsPath  []string
-	fuseMount        []string
-	apptainerEnv     map[string]string
-	apptainerEnvFile string
-	noMount          []string
-	dmtcpLaunch      string
-	dmtcpRestart     string
+	appName           string
+	bindPaths         []string
+	mounts            []string
+	homePath          string
+	overlayPath       []string
+	scratchPath       []string
+	workdirPath       string
+	cwdPath           string
+	shellPath         string
+	hostname          string
+	network           string
+	networkArgs       []string
+	dns               string
+	security          []string
+	cgroupsTOMLFile   string
+	containLibsPath   []string
+	fuseMount         []string
+	apptainerEnv      map[string]string
+	apptainerEnvFiles []string
+	noMount           []string
+	dmtcpLaunch       string
+	dmtcpRestart      string
 
 	isBoot          bool
 	isFakeroot      bool
@@ -60,6 +60,7 @@ var (
 	disableCache    bool
 
 	netNamespace   bool
+	netnsPath      string
 	utsNamespace   bool
 	userNamespace  bool
 	pidNamespace   bool
@@ -609,8 +610,8 @@ var actionEnvFlag = cmdline.Flag{
 // --env-file
 var actionEnvFileFlag = cmdline.Flag{
 	ID:           "actionEnvFileFlag",
-	Value:        &apptainerEnvFile,
-	DefaultValue: "",
+	Value:        &apptainerEnvFiles,
+	DefaultValue: []string{},
 	Name:         "env-file",
 	Usage:        "pass environment variables from file to contained process",
 	EnvKeys:      []string{"ENV_FILE"},
@@ -843,6 +844,16 @@ var actionRunscriptTimeoutFlag = cmdline.Flag{
 	Hidden:       false,
 }
 
+// --netns-path
+var actionNetnsPathFlag = cmdline.Flag{
+	ID:           "actionNetnsPathFlag",
+	Value:        &netnsPath,
+	DefaultValue: "",
+	Name:         "netns-path",
+	Usage:        "join the network namespace at the specified path (as root, or if permitted in apptainer.conf)",
+	EnvKeys:      []string{"NETNS_PATH"},
+}
+
 func init() {
 	addCmdInit(func(cmdManager *cmdline.CommandManager) {
 		cmdManager.RegisterCmd(ExecCmd)
@@ -885,6 +896,7 @@ func init() {
 		cmdManager.RegisterFlagForCmd(&actionKeepPrivsFlag, actionsInstanceCmd...)
 		cmdManager.RegisterFlagForCmd(&actionMountFlag, actionsInstanceCmd...)
 		cmdManager.RegisterFlagForCmd(&actionNetNamespaceFlag, actionsInstanceCmd...)
+		cmdManager.RegisterFlagForCmd(&actionNetnsPathFlag, actionsInstanceCmd...)
 		cmdManager.RegisterFlagForCmd(&actionNetworkArgsFlag, actionsInstanceCmd...)
 		cmdManager.RegisterFlagForCmd(&actionNetworkFlag, actionsInstanceCmd...)
 		cmdManager.RegisterFlagForCmd(&actionNoHomeFlag, actionsInstanceCmd...)
@@ -939,6 +951,7 @@ func init() {
 		cmdManager.RegisterFlagForCmd(&actionIgnoreUsernsFlag, actionsInstanceCmd...)
 		cmdManager.RegisterFlagForCmd(&actionUnderlayFlag, actionsInstanceCmd...)
 		cmdManager.RegisterFlagForCmd(&actionShareNSFlag, actionsCmd...)
+		cmdManager.RegisterFlagForCmd(&commonAuthFileFlag, actionsInstanceCmd...)
 		cmdManager.RegisterFlagForCmd(&actionRunscriptTimeoutFlag, actionsRunscriptCmd...)
 	})
 }
