@@ -2,7 +2,7 @@
 //   Apptainer a Series of LF Projects LLC.
 //   For website terms of use, trademark policy, privacy policy and other
 //   project policies see https://lfprojects.org/policies
-// Copyright (c) 2019-2022, Sylabs Inc. All rights reserved.
+// Copyright (c) 2019-2023, Sylabs Inc. All rights reserved.
 // This software is licensed under a 3-clause BSD license. Please consult the
 // LICENSE.md file distributed with the sources of this project regarding your
 // rights to use or distribute this software.
@@ -62,7 +62,6 @@ func (c imgBuildTests) buildFrom(t *testing.T) {
 	// see https://github.com/apptainer/singularity/issues/4407
 	tt := []struct {
 		name         string
-		dependency   string
 		buildSpec    string
 		requirements func(t *testing.T)
 	}{
@@ -105,8 +104,8 @@ func (c imgBuildTests) buildFrom(t *testing.T) {
 			buildSpec: c.env.OrasTestImage,
 		},
 		{
-			name:      "Yum CentOS7",
-			buildSpec: "../examples/centos/Apptainer",
+			name:      "Yum AlmaLinux 9",
+			buildSpec: "../examples/almalinux-amd64/Apptainer",
 			requirements: func(t *testing.T) {
 				require.Command(t, "yum")
 				require.RPMMacro(t, "_db_backend", "bdb")
@@ -114,20 +113,9 @@ func (c imgBuildTests) buildFrom(t *testing.T) {
 				require.Arch(t, "amd64")
 			},
 		},
-		{
-			name:       "YumArm64 CentOS 7",
-			dependency: "yum",
-			buildSpec:  "../examples/centos-arm64/Apptainer",
-			requirements: func(t *testing.T) {
-				require.Command(t, "yum")
-				require.RPMMacro(t, "_db_backend", "bdb")
-				require.RPMMacro(t, "_dbpath", "/var/lib/rpm")
-				require.Arch(t, "arm64")
-			},
-		},
 		{
 			name:      "Dnf AlmaLinux 9",
-			buildSpec: "../examples/almalinux/Apptainer",
+			buildSpec: "../examples/almalinux-amd64/DnfDef",
 			requirements: func(t *testing.T) {
 				require.Command(t, "dnf")
 				require.RPMMacro(t, "_db_backend", "sqlite")
@@ -136,9 +124,9 @@ func (c imgBuildTests) buildFrom(t *testing.T) {
 			},
 		},
 		{
-			name:       "DnfArm64 AlmaLinux 9",
-			dependency: "yum",
-			buildSpec:  "../examples/almalinux-arm64/Apptainer",
+			name: "DnfArm64 AlmaLinux 9",
+			// dependency: "yum",
+			buildSpec: "../examples/almalinux-arm64/Apptainer",
 			requirements: func(t *testing.T) {
 				require.Command(t, "dnf")
 				require.RPMMacro(t, "_db_backend", "sqlite")
@@ -148,7 +136,7 @@ func (c imgBuildTests) buildFrom(t *testing.T) {
 		},
 		{
 			name:      "Dnf Fedora 37",
-			buildSpec: "../examples/fedora/Apptainer",
+			buildSpec: "../examples/fedora-amd64/Apptainer",
 			requirements: func(t *testing.T) {
 				require.Command(t, "dnf")
 				require.RPMMacro(t, "_db_backend", "sqlite")
@@ -157,9 +145,9 @@ func (c imgBuildTests) buildFrom(t *testing.T) {
 			},
 		},
 		{
-			name:       "DnfArm64 Fedora 37",
-			dependency: "yum",
-			buildSpec:  "../examples/fedora-arm64/Apptainer",
+			name: "DnfArm64 Fedora 37",
+			// dependency: "yum",
+			buildSpec: "../examples/fedora-arm64/Apptainer",
 			requirements: func(t *testing.T) {
 				require.Command(t, "dnf")
 				require.RPMMacro(t, "_db_backend", "sqlite")
@@ -169,7 +157,7 @@ func (c imgBuildTests) buildFrom(t *testing.T) {
 		},
 		{
 			name:      "Zypper",
-			buildSpec: "../examples/opensuse/Apptainer",
+			buildSpec: "../examples/opensuse-amd64/Apptainer",
 			requirements: func(t *testing.T) {
 				require.Command(t, "zypper")
 				require.Arch(t, "amd64")
@@ -396,6 +384,41 @@ func (c imgBuildTests) buildLocalImage(t *testing.T) {
 	}
 }
 
+func (c imgBuildTests) buildExtraArgs(t *testing.T) {
+	e2e.EnsureImage(t, c.env)
+
+	tmpdir, cleanup := c.tempDir(t, "build-extra-args")
+
+	t.Cleanup(func() {
+		if !t.Failed() {
+			cleanup()
+		}
+	})
+	liDefFile := e2e.PrepareDefFile(e2e.DefFileDetails{
+		Bootstrap: "localimage",
+		From:      c.env.ImagePath,
+	})
+	t.Cleanup(func() {
+		if !t.Failed() {
+			os.Remove(liDefFile)
+		}
+	})
+
+	img := path.Join(tmpdir, "test.sif")
+
+	c.env.RunApptainer(
+		t,
+		e2e.WithProfile(e2e.UserNamespaceProfile),
+		e2e.WithCommand("build"),
+		e2e.WithArgs("--mksquashfs-args", "-comp lz4", img, liDefFile),
+		e2e.WithEnv(append(os.Environ(), "APPTAINER_VERBOSE=true")),
+		e2e.ExpectExit(
+			0,
+			e2e.ExpectOutput(e2e.ContainMatch, "lz4 compressed"),
+		),
+	)
+}
+
 func (c imgBuildTests) badPath(t *testing.T) {
 	dn, cleanup := c.tempDir(t, "bad-path")
 	t.Cleanup(func() {
@@ -1429,7 +1452,7 @@ func (c imgBuildTests) buildUpdateSandbox(t *testing.T) {
 	testDir, cleanup := e2e.MakeTempDir(t, c.env.TestDir, "build-sandbox-", "")
 	t.Cleanup(func() {
 		if !t.Failed() {
-			e2e.Privileged(cleanup)
+			e2e.Privileged(cleanup)(t)
 		}
 	})
 
@@ -1812,7 +1835,7 @@ func (c imgBuildTests) buildBindMount(t *testing.T) {
 			e2e.WithProfile(e2e.RootProfile),
 			e2e.WithCommand("build"),
 			e2e.WithArgs(args...),
-			e2e.PostRun(func(t *testing.T) {
+			e2e.PostRun(func(_ *testing.T) {
 				os.Remove(defFile)
 			}),
 			e2e.ExpectExit(tt.exit),
@@ -1841,7 +1864,7 @@ func (c imgBuildTests) buildLibraryHost(t *testing.T) {
 		e2e.WithProfile(e2e.RootProfile),
 		e2e.WithCommand("build"),
 		e2e.WithArgs("-F", imagePath, defFile),
-		e2e.PostRun(func(t *testing.T) {
+		e2e.PostRun(func(_ *testing.T) {
 			os.Remove(defFile)
 		}),
 		e2e.ExpectExit(255,
@@ -1872,7 +1895,7 @@ func (c imgBuildTests) testWritableTmpfs(t *testing.T) {
 		e2e.WithProfile(e2e.RootProfile),
 		e2e.WithCommand("build"),
 		e2e.WithArgs("-F", "--writable-tmpfs", imagePath, defFile),
-		e2e.PostRun(func(t *testing.T) {
+		e2e.PostRun(func(_ *testing.T) {
 			os.Remove(defFile)
 		}),
 		e2e.ExpectExit(0),
@@ -2105,7 +2128,7 @@ cat /proc/$$/cmdline`
 		e2e.WithProfile(e2e.RootProfile),
 		e2e.WithCommand("build"),
 		e2e.WithArgs("-F", imagePath, defFile),
-		e2e.PostRun(func(t *testing.T) {
+		e2e.PostRun(func(_ *testing.T) {
 			os.Remove(defFile)
 		}),
 		e2e.ExpectExit(0,
@@ -2232,17 +2255,120 @@ func (c imgBuildTests) testGocryptfsSIFBuild(t *testing.T) {
 	}
 }
 
+func (c imgBuildTests) buildWithAuth(t *testing.T) {
+	e2e.EnsureImage(t, c.env)
+
+	profiles := []e2e.Profile{
+		e2e.FakerootProfile,
+		e2e.RootProfile,
+	}
+
+	for _, p := range profiles {
+		t.Run(p.String(), func(t *testing.T) {
+			t.Run("default", func(t *testing.T) {
+				c.buildWithAuthTester(t, false, p)
+			})
+			t.Run("custom", func(t *testing.T) {
+				c.buildWithAuthTester(t, true, p)
+			})
+		})
+	}
+}
+
+func (c imgBuildTests) buildWithAuthTester(t *testing.T, withCustomAuthFile bool, profile e2e.Profile) {
+	e2e.EnsureImage(t, c.env)
+
+	simpleDef := e2e.PrepareDefFile(e2e.DefFileDetails{
+		Bootstrap: "docker",
+		From:      strings.TrimPrefix(c.env.TestRegistryPrivImage, "docker://"),
+	})
+	t.Cleanup(func() {
+		if !t.Failed() {
+			os.Remove(simpleDef)
+		}
+	})
+
+	tmpdir, tmpdirCleanup := e2e.MakeTempDir(t, c.env.TestDir, "build-auth", "")
+	t.Cleanup(func() {
+		if !t.Failed() {
+			tmpdirCleanup(t)
+		}
+	})
+
+	prevCwd, err := os.Getwd()
+	if err != nil {
+		t.Fatalf("could not get current working directory: %s", err)
+	}
+	defer os.Chdir(prevCwd)
+	if err = os.Chdir(tmpdir); err != nil {
+		t.Fatalf("could not change cwd to %q: %s", tmpdir, err)
+	}
+
+	localAuthFileName := ""
+	if withCustomAuthFile {
+		localAuthFileName = "./my_local_authfile"
+	}
+
+	authFileArgs := []string{}
+	if withCustomAuthFile {
+		authFileArgs = []string{"--authfile", localAuthFileName}
+	}
+
+	t.Cleanup(func() {
+		e2e.PrivateRepoLogout(t, c.env, profile, localAuthFileName)
+	})
+
+	tests := []struct {
+		name          string
+		args          []string
+		whileLoggedIn bool
+		expectExit    int
+	}{
+		{
+			name:          "privimg logged out",
+			args:          []string{"-F", "--no-https", "--disable-cache", "./my_image_file.sif", simpleDef},
+			whileLoggedIn: false,
+			expectExit:    255,
+		},
+		{
+			name:          "privimg logged in",
+			args:          []string{"-F", "--no-https", "--disable-cache", "./my_image_file.sif", simpleDef},
+			whileLoggedIn: true,
+			expectExit:    0,
+		},
+	}
+
+	for _, tt := range tests {
+		if tt.whileLoggedIn {
+			e2e.PrivateRepoLogin(t, c.env, profile, localAuthFileName)
+		} else {
+			e2e.PrivateRepoLogout(t, c.env, profile, localAuthFileName)
+		}
+		c.env.RunApptainer(
+			t,
+			e2e.AsSubtest(tt.name),
+			e2e.WithProfile(profile),
+			e2e.WithCommand("build"),
+			e2e.WithArgs(append(authFileArgs, tt.args...)...),
+			e2e.ExpectExit(tt.expectExit),
+		)
+	}
+}
+
 // E2ETests is the main func to trigger the test suite
 func E2ETests(env e2e.TestEnv) testhelper.Tests {
 	c := imgBuildTests{
 		env: env,
 	}
 
+	np := testhelper.NoParallel
+
 	return testhelper.Tests{
 		"bad path":                               c.badPath,                              // try to build from a non existent path
 		"build encrypt with PEM file":            c.buildEncryptPemFile,                  // build encrypted images with certificate
 		"build encrypted with passphrase":        c.buildEncryptPassphrase,               // build encrypted images with passphrase
 		"definition":                             c.buildDefinition,                      // builds from definition template
+		"extra mksquashfs args":                  c.buildExtraArgs,                       // build with extra mksquashfs args
 		"from local image":                       c.buildLocalImage,                      // build and image from an existing image
 		"from":                                   c.buildFrom,                            // builds from definition file and URI
 		"multistage":                             c.buildMultiStageDefinition,            // multistage build from definition templates
@@ -2274,5 +2400,7 @@ func E2ETests(env e2e.TestEnv) testhelper.Tests {
 		"build sif image using gocryptfs":        c.testGocryptfsSIFBuild,                // https://github.com/apptainer/apptainer/issues/484
 		"definition build with template support": c.buildDefinitionWithBuildArgs,         // builds from definition with build args (build arg file) support
 		"issue 1812":                             c.issue1812,                            // https://github.com/sylabs/singularity/issues/1812
+		"auth":                                   np(c.buildWithAuth),                    // build with custom auth file
+		"issue 2607":                             c.issue2607,                            // https://github.com/sylabs/singularity/issues/2607
 	}
 }
