@@ -2,7 +2,7 @@
 //   Apptainer a Series of LF Projects LLC.
 //   For website terms of use, trademark policy, privacy policy and other
 //   project policies see https://lfprojects.org/policies
-// Copyright (c) 2022-2024, Sylabs Inc. All rights reserved.
+// Copyright (c) 2022-2025, Sylabs Inc. All rights reserved.
 // This software is licensed under a 3-clause BSD license. Please consult the
 // LICENSE.md file distributed with the sources of this project regarding your
 // rights to use or distribute this software.
@@ -10,10 +10,12 @@
 package cgroups
 
 import (
+	"errors"
 	"fmt"
 	"os"
 	"strings"
 
+	"github.com/apptainer/apptainer/internal/pkg/util/fs"
 	"github.com/apptainer/apptainer/pkg/util/namespaces"
 	"github.com/opencontainers/runc/libcontainer/cgroups"
 	"golang.org/x/sys/unix"
@@ -96,6 +98,7 @@ func HasXDGRuntimeDir() (bool, error) {
 }
 
 // CanUseCgroups checks whether it's possible to use the cgroups manager.
+// - Systemd cgroups management requires systemd running as init.
 // - Host root can always use cgroups.
 // - Rootless needs cgroups v2.
 // - Rootless needs systemd manager.
@@ -105,20 +108,26 @@ func HasXDGRuntimeDir() (bool, error) {
 // Returns nil if can be used, otherwise returns an error explaining why
 // it can't be used
 func CanUseCgroups(systemd bool) error {
+	if systemd {
+		systemdRunning := fs.IsDir("/run/systemd/system")
+		if !systemdRunning {
+			return errors.New("cannot use systemd cgroups manager, systemd not running as init on this host")
+		}
+	}
 	uid := os.Geteuid()
 	if uid == 0 {
 		if !namespaces.IsUnprivileged() {
 			return nil
 		}
-		return fmt.Errorf("rootless cgroups is not usable in fakeroot mode")
+		return errors.New("rootless cgroups is not usable in fakeroot mode")
 	}
 
 	if !cgroups.IsCgroup2UnifiedMode() {
-		return fmt.Errorf("system is not configured for cgroups v2 in unified mode")
+		return errors.New("system is not configured for cgroups v2 in unified mode")
 	}
 
 	if !systemd {
-		return fmt.Errorf("'systemd cgroups' is not enabled in apptainer.conf")
+		return errors.New("'systemd cgroups' is not enabled in apptainer.conf")
 	}
 
 	if ok, err := HasDbus(); !ok {
