@@ -20,28 +20,28 @@ var Manager = new(manager)
 type manager struct{}
 
 // Login allows to log into a service like a Docker/OCI registry or a keyserver.
-func (m *manager) Login(uri, username, password string, insecure bool) (*Config, error) {
+func (m *manager) Login(uri, username, password string, insecure bool, reqAuthFile string) (*Config, error) {
 	u, err := url.Parse(uri)
 	if err != nil {
 		return nil, err
 	}
 
 	if handler, ok := loginHandlers[u.Scheme]; ok {
-		return handler.login(u, username, password, insecure)
+		return handler.login(u, username, password, insecure, reqAuthFile)
 	}
 
 	return nil, fmt.Errorf("%s transport is not supported", u.Scheme)
 }
 
 // Logout allows to log out from a service like a Docker/OCI registry or a keyserver.
-func (m *manager) Logout(uri string) error {
+func (m *manager) Logout(uri string, reqAuthFile string) error {
 	u, err := url.Parse(uri)
 	if err != nil {
 		return err
 	}
 
 	if handler, ok := loginHandlers[u.Scheme]; ok {
-		return handler.logout(u)
+		return handler.logout(u, reqAuthFile)
 	}
 
 	return fmt.Errorf("%s transport is not supported", u.Scheme)
